//------------------------------------------------------
//
//   Greg's Blogging - SMTP
//
//------------------------------------------------------

package gcewing.blogging;

import java.net.*;
import java.io.*;
import java.util.*;
import sun.misc.BASE64Encoder;

import net.minecraftforge.common.*;

public class SMTP {

	public static boolean smtpEnabled;
	public static boolean emailEnabled;
	static String server;
	static int port;
	static String username;
	static String password;
	static String fromAddress;
	static String[] toAddresses;
	static String hostname;
	static boolean debug;
	static boolean debugLots = true;

	BASE64Encoder base64 = new BASE64Encoder();
	Random random = new Random();

	public static void configure(BaseConfiguration config) {
		smtpEnabled = config.getBoolean("smtp", "enabled", true);
		server = config.getString("smtp", "server", "");
		port = config.getInteger("smtp", "port", 25);
		username = config.getString("smtp", "username", "");
		password = config.getString("smtp", "password", "");
		hostname = config.getString("smtp", "hostname", "minecraft");
		debug = config.getBoolean("smtp", "debug", false);
		emailEnabled = config.getBoolean("email", "enabled", true);
		fromAddress = config.getString("email", "from", "");
		String toString = config.getString("email", "to", "");
		if (!toString.equals(""))
			toAddresses = toString.replace(";", ",").split(",");
		else
			toAddresses = new String[0];
		System.out.printf("GregsBlogging.SMTP.server = %s\n", server);
		System.out.printf("GregsBlogging.SMTP.port = %s\n", port);
		System.out.printf("GregsBlogging.SMTP.username = %s\n", username);
		System.out.printf("GregsBlogging.SMTP.debug = %s\n", debug);
		System.out.printf("GregsBlogging.SMTP.fromAddress = %s\n", fromAddress);
		System.out.printf("GregsBlogging.SMTP.toAddresses = [");
		for (String a : toAddresses)
			System.out.print(a);
		System.out.printf("]\n");
		System.out.printf("GregsBlogging.SMTP.hostname = %s\n", hostname);
	}
	
	public static boolean isEnabled() {
		return smtpEnabled && emailEnabled && toAddresses != null && toAddresses.length != 0;
	}

	DataOutputStream out;
	DataInputStream inp;
	String boundary;

	public SMTP(String subject, String html, byte[] image) {
		try {
			Socket socket = new Socket(server, port); 
			out = new DataOutputStream(socket.getOutputStream());
			inp = new DataInputStream(socket.getInputStream());
			boundary = format("Boundary_%08x%08x", random.nextInt(), random.nextInt());
			getResponse();
			command("HELO %s", hostname);
			if (!username.equals("")) {
				String b64username = base64.encode(username.getBytes());
				String b64password = base64.encode(password.getBytes());
				command("AUTH LOGIN");
				command("%s", b64username);
				command("%s", b64password);
			}
			command("MAIL FROM: %s", fromAddress);
			for (String toAddress : toAddresses)
				command("RCPT TO: %s", toAddress);
			command("DATA");
			for (String toAddress : toAddresses)
				writeLn("To: %s", toAddress);
			writeLn("Subject: %s", subject);
			writeLn("Content-Type: multipart/related; boundary=\"%s\"", boundary);
			writeLn("MIME-Version: 1.0");
			writeLn("");
			//System.out.printf("SMTP< [%d data bytes]", content.length());
			//sendContent(content);
			writeQuotedPrintablePart("text/html", html);
			if (image != null)
				writeBase64Part("screenshot", "image/jpeg", "screenshot.jpg", image);
				//writeBase64Part("screenshot", "image/png", "screenshot.png", image);
			writeMimeFooter();
			command(".");
			command("QUIT\r\n");
			inp.close();
			out.close();		       
			socket.close();
		}
		catch (IOException e) {
			//throw new EnvironmentError(e.getMessage());
			throw new EnvironmentError(e.getClass().getSimpleName() + ": " + e.getMessage());
		}
	}
	
	void writeQuotedPrintablePart(String type, String content) throws IOException {
		echo("[%d bytes of text]", content.length());
		writeLn("--%s", boundary);
		writeLn("Content-Type: %s", type);
		writeLn("Content-Transfer-Encoding: quoted-printable");
		writeLn("");
		int n = content.length();
		int i = 0;
		while (i < n) {
			int j = i + 75;
			if (j > n)
				j = n;
			String line = content.substring(i, j);
			line = line.replace("=", "=3D");
			line = line.replace("\n.", "\n..");
			line = line.replace("\n", "\r\n");
			if (line.charAt(0) == '.')
				line = "." + line;
			if (j < n)
				line = line + "=";
			writeLn(line);
			i = j;
		}
		writeLn("");
	}
	
	void writeBase64Part(String id, String type, String filename, byte[] data) throws IOException {
		echo("[%d bytes of data]", data.length);
		writeLn("--%s", boundary);
		writeLn("Content-ID: <%s>", id);
		writeLn("Content-Type: %s", type);
		writeLn("Content-Transfer-Encoding: BASE64");
		writeLn("Content-Disposition: inline; filename=\"%s\"", filename);
		writeLn("");
		writeLn(base64.encode(data).replace("\n", "\r\n"));
	}
	
	void writeMimeFooter() throws IOException {
		writeLn("--%s--", boundary);
	}
	
	void command(String fmt, Object... args) throws IOException {
		send(fmt, args);
		getResponse();
	}

	void send(String fmt, Object... args) throws IOException {
		echo(fmt, args);
		writeLn(fmt, args);
	}
	
	void writeLn(String fmt, Object... args) throws IOException {
		write(fmt, args);
		out.writeBytes("\r\n");
	}
	
	void write(String fmt, Object... args) throws IOException {
		String data = format(fmt, args);
		if (debug && debugLots)
			System.out.printf("SMTP< %s\n", data);
		out.writeBytes(data);
	}
			
	void echo(String fmt, Object... args) {
		if (debug && !debugLots)
			System.out.printf("SMTP< %s\n", format(fmt, args));
	}
	
	static String format(String fmt, Object... args) {
		Formatter f = new Formatter();
		f.format(fmt, args);
		return ((StringBuilder)f.out()).toString();
	}

	void getResponse() throws IOException {
		out.flush();
		String response = inp.readLine();
		if (debug) {
			if (response != null)
				System.out.printf("SMTP> %s\n", response);
			else
				System.out.printf("SMTP> EOF\n");
		}
		Scanner scanner = new Scanner(response);
		int code = scanner.nextInt();
		//System.out.printf("SMTP Response Code = %d\n", code);
		if (code >= 400) {
			IOException e = new SMTPError(response);
			throw e;
		}
	}
	
	static class SMTPError extends IOException {
	
		SMTPError(String s) {
			super(s);
		}

	}

}
