//------------------------------------------------------------------------------
//
//   Greg's Blogging Mod - XML document
//
//------------------------------------------------------------------------------

package gcewing.blogging.web;

import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
 
import org.w3c.dom.*;
import org.xml.sax.*;

import gcewing.blogging.*;

public class XML {

	static DocumentBuilderFactory dbf;
	static DocumentBuilder db;
	static TransformerFactory trf;
	static Transformer tr;
	
	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			db = dbf.newDocumentBuilder();
			trf = TransformerFactory.newInstance();
			tr = trf.newTransformer();
			tr.setOutputProperty(OutputKeys.INDENT, "yes");
			tr.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
		}
		catch (Exception e) {
			throw new EnvironmentError(e);
		}
	}

	Document doc;
	Element root;
	
	public static XML parse(String data) {
		try {
			Document doc = db.parse(new InputSource(new StringReader(data)));
			return new XML(doc);
		}
		catch (Exception e) {
			throw new EnvironmentError(e);
		}
	}
	
	public XML(Document doc) {
		this.doc = doc;
		this.root = (Element)doc.getFirstChild();
	}
	
	public XML(String rootName) {
		doc = db.newDocument();
		root = elem(rootName);
		doc.appendChild(root);
	}

	Element elem(String name, Node... children) {
		Element node = doc.createElement(name);
		add(node, children);
		return node;
	}
	
	Element elem(String name, String text) {
		Element node = doc.createElement(name);
		Text contents = doc.createTextNode(text);
		node.appendChild(contents);
		return node;
	}
	
	void add(Node node, Node... children) {
		for (Node child : children)
			node.appendChild(child);
	}
	
	public String toString() {
		try {
			DOMSource src = new DOMSource(doc);
			StringWriter sw = new StringWriter();
			StreamResult dst = new StreamResult(sw);
			tr.transform(src, dst);
			return sw.toString();
		}
		catch (Exception e) {
			throw new EnvironmentError(e);
		}
	}

}
